#include <types.h>
#include <system_def.h>
#include <tpu.h>


/********************************************************************/
/* IRC from TPU  */

void tpu_set_ier(chan,val)
{
  if(val)
    atomic_set_mask_w(1<<chan,TPU_CIER);  
  else
    atomic_clear_mask_w(1<<chan,TPU_CIER);  
}


void tpu_set_cfsr(int chan,int val)
{
  atomic_clear_mask_w(0xf<<((chan&3)<<2),TPU_CFSR0+3-(chan>>2));
  atomic_set_mask_w((val&0xf)<<((chan&3)<<2),TPU_CFSR0+3-(chan>>2));  
}


void tpu_set_hsqr(int chan,int val)
{
  atomic_clear_mask_w(~(val&3)<<((chan&7)<<1),TPU_HSQR0+1-(chan>>3));
  atomic_set_mask_w((val&3)<<((chan&7)<<1),TPU_HSQR0+1-(chan>>3));
}


void tpu_set_hsrr(int chan,int val)
{
  atomic_clear_mask_w(3<<((chan&7)<<1),TPU_HSRR0+1-(chan>>3));
  atomic_set_mask_w((val&3)<<((chan&7)<<1),TPU_HSRR0+1-(chan>>3));  
}


void tpu_set_cpr(int chan,int val)
{
  atomic_clear_mask_w(3<<((chan&7)<<1),TPU_CPR0+1-(chan>>3));
  atomic_set_mask_w((val&3)<<((chan&7)<<1),TPU_CPR0+1-(chan>>3));  
}


inline volatile __u16 * tpu_param_addr(int chan,int par)
{
  return TPU_PARAM+8*chan+par;
}


#define EDGE_TIME          0
#define POSITION_COUNT     1
#define TCR1_VALUE         2
#define CHAN_PINSTATE      3
#define CORR_PINSTATE_ADDR 4
#define EDGE_TIME_LSB_ADDR 5

int tpu_irc_init(int chan)
{
  if(chan>15) return -1;
  
  tpu_set_cpr(chan,0);		/* disable chanel function */
  tpu_set_cpr(chan+1,0);	/* disable chanel function */
  tpu_set_cfsr(chan,0x6);	/* FQD - Fast Quadrature Decode */
  tpu_set_cfsr(chan+1,0x6);	/* FQD - Fast Quadrature Decode */
  tpu_set_hsqr(chan,0x0);	/* Primary Chanel */
  tpu_set_hsqr(chan+1,0x1);	/* Secondary Chanel */
  
  *tpu_param_addr(chan,EDGE_TIME_LSB_ADDR)=
          ((int)tpu_param_addr(chan,EDGE_TIME)&0xff)+1;
  *tpu_param_addr(chan+1,EDGE_TIME_LSB_ADDR)=
          ((int)tpu_param_addr(chan,EDGE_TIME)&0xff)+1;
  *tpu_param_addr(chan,CORR_PINSTATE_ADDR)=
          (int)tpu_param_addr(chan+1,CHAN_PINSTATE)&0xff;
  *tpu_param_addr(chan+1,CORR_PINSTATE_ADDR)=
          (int)tpu_param_addr(chan,CHAN_PINSTATE)&0xff;

  *tpu_param_addr(chan,POSITION_COUNT)=0;
  tpu_set_cpr(chan,2);		/* medium priority */
  tpu_set_cpr(chan+1,2);	/* medium priority */
  tpu_set_hsrr(chan,3);
  tpu_set_hsrr(chan+1,3);

  return 0;
}

int tpu_irc_read(int chan)
{
  return *tpu_param_addr(chan,POSITION_COUNT);
}

#if 0
void tpu_irc_update(int chan, mo_reg_state_t *reg)
{
  int irc;
  int spd;
  int val_mag=1<<(reg->mrs_subdiv+16);
  irc=*tpu_param_addr(chan,POSITION_COUNT);
  irc<<=reg->mrs_subdiv;
  irc|=reg->mrs_ap&~(val_mag-1);
  spd=irc-reg->mrs_ap;
  if(spd<-(val_mag>>1)) {irc+=val_mag;spd+=val_mag;}
  else if(spd>(val_mag>>1)) {irc-=val_mag;spd-=val_mag;}
  reg->mrs_ap=irc;
  reg->mrs_as=spd;
}
#endif
